@@ -9,11 +9,11 @@
 #' @param cases 0 based vector of case indices with respect to y.
 #' @param counts Vector of variant counts.
 #' @template min_ac
-#' @param q_shape Beta distribution parameterisation of benign variant configuration rate of affection, q.
-#' @param p_shape Beta distribution parameterisation of pathogenic variant configuration rate of affection, p.
+#' @param tau_shape Beta distribution parameterisation of benign variant configuration rate of affection, q.
+#' @param pi_shape Beta distribution parameterisation of pathogenic variant configuration rate of affection, p.
 #' @param omega_shape Beta distribution of global rate of pathogenicty of variants in gene given pathogenicity of gene, omega.
 #' @template temperatures
-#' @param Z0_matrix Matrix of logicals, where the rows are used as an initial Zs for the chains.
+#' @param z0_matrix Matrix of logicals, where the rows are used as an initial zs for the chains.
 #' @param estimate_omega Logical value determining whether to estimate the parameter omega.
 #' @param logit_omegas Numeric vector of logit omega values, one value per chain.
 #' @param logit_omega_proposal_sds Numeric vector of proposal standard deviations for Metropolis-Hastings sampling of logit omega parameter, one value per chain.
@@ -29,7 +29,8 @@
 #' @param case_variant_block_starts 0-indexed start positions for contiguous blocks of variants in \code{case_variants}.
 #' @param case_variant_block_ends As \code{case_variant_block_starts} for (exclusive) stop positions.
 #' @param case_variants Integer vector giving variant numbers (0-based, i.e. between 0 and k-1). Used to pick pairs of variants for tandem updates from.
-#' @template store_Z_trace
+#' @template return_z_trace
+#' @template return_x_trace
 #' @template burn
 #' @param check Logical value indicating whether to perform validation on the arguments before calling the c++ function. 
 #' @return Object of class \code{BeviMed}, containing the output of the MCMC sampling.
@@ -43,11 +44,11 @@ call_cpp <- function(
 	cases,
 	counts,
 	min_ac,
-	q_shape,
-	p_shape,
+	tau_shape,
+	pi_shape,
 	omega_shape,
 	temperatures,
-	Z0_matrix,
+	z0_matrix,
 	estimate_omega,
 	logit_omegas,
 	logit_omega_proposal_sds,
@@ -63,16 +64,18 @@ call_cpp <- function(
 	case_variant_block_starts,
 	case_variant_block_ends,
 	case_variants,
-	store_Z_trace,
+	return_z_trace,
+	return_x_trace,
 	burn=0,
 	check=TRUE
 ) {
 	if (check) {
 		stopifnot(length(omega_shape) == 2 & min(omega_shape) > 0)
-		stopifnot(length(q_shape) == 2 & min(q_shape) > 0)
-		stopifnot(length(p_shape) == 2 & min(p_shape) > 0)
-		stopifnot(is.matrix(Z0_matrix))
-		stopifnot(length(temperatures) == nrow(Z0_matrix))
+		stopifnot(length(tau_shape) == 2 & min(tau_shape) > 0)
+		stopifnot(length(pi_shape) == 2 & min(pi_shape) > 0)
+		stopifnot(is.matrix(z0_matrix))
+		stopifnot(ncol(z0_matrix) == length(variant_weights))
+		stopifnot(length(temperatures) == nrow(z0_matrix))
 		stopifnot(length(temperatures) == nrow(logit_omegas))
 		stopifnot(length(temperatures) == nrow(log_phis))
 		stopifnot(length(temperatures) == length(logit_omega_proposal_sds))
@@ -84,8 +87,7 @@ call_cpp <- function(
 		if (tandem_variant_updates > 0 & length(unique(case_variants)) < 2) stop("Must have more than 1 variant to select from if making tandem updates")
 	}
 
-	raw <- .Call(
-		"R_bevimed_mc",
+	raw <- bevimed_mc(
 		samples_per_chain,
 		y,
 		block_starts,
@@ -93,13 +95,13 @@ call_cpp <- function(
 		cases,
 		counts,
 		if (length(min_ac) == length(y)) min_ac else rep(min_ac, length(y)),
-		q_shape[1],
-		q_shape[2],
-		p_shape[1],
-		p_shape[2],
+		tau_shape[1],
+		tau_shape[2],
+		pi_shape[1],
+		pi_shape[2],
 		omega_shape[1],
 		omega_shape[2],
-		Z0_matrix,
+		z0_matrix,
 		estimate_omega,
 		logit_omegas,
 		logit_omega_proposal_sds,
@@ -116,8 +118,8 @@ call_cpp <- function(
 		case_variant_block_starts,
 		case_variant_block_ends,
 		case_variants,
-		store_Z_trace,
-		PACKAGE="BeviMed"
+		return_z_trace,
+		return_x_trace
 	)
 
 	if (burn > 0) {
@@ -126,7 +128,8 @@ call_cpp <- function(
 				raw[c(
 					"y_log_lik",
 					"y_log_lik_t_equals_1",
-					"Z",
+					"z",
+					"x",
 					"logit_omega",
 					"log_phi"
 				)],
@@ -136,7 +139,7 @@ call_cpp <- function(
 				raw[c("swap_accept", "swap_at_temperature")],
 				function(x) x[-seq(length.out=chain_swaps_per_cycle * burn)]
 			),
-			raw[c("terminal_Z", "terminal_log_phi", "terminal_logit_omega")]
+			raw[c("terminal_z", "terminal_log_phi", "terminal_logit_omega")]
 		)
 	}
 
@@ -145,8 +148,8 @@ call_cpp <- function(
 			raw,
 			list(
 				omega_shape=omega_shape,
-				p_shape=p_shape,
-				q_shape=q_shape,
+				pi_shape=pi_shape,
+				tau_shape=tau_shape,
 				variant_weights=variant_weights,
 				temperatures=temperatures,
 				estimate_phi=estimate_phi,
@@ -154,12 +157,12 @@ call_cpp <- function(
 				y=y,
 				min_ac=min_ac,
 				variant_table=data.frame(
-					variant=unlist(mapply(SIMPLIFY=FALSE, FUN=rep, 1:length(block_starts), block_ends-block_starts)),
+					variant=unlist(mapply(SIMPLIFY=FALSE, FUN=rep, seq(length(block_starts)), block_ends-block_starts)),
 					case=cases+1,
 					count=counts
 				),
 				n=length(y),
-				k=ncol(Z0_matrix)
+				k=ncol(z0_matrix)
 			)
 		),
 		class="BeviMed"
@@ -172,7 +175,7 @@ call_cpp <- function(
 #' @param temperatures Numeric vector of temperatures used to produce \code{y_log_lik_t_equals_1_traces}.
 #' @return Numeric value of estimated log marginal likelihood.
 sum_ML_over_PP <- function(y_log_lik_t_equals_1_traces, temperatures) {
-	sum(mapply(FUN=function(y_lik, t_diff) { log(mean(exp(t_diff*(y_lik)))) }, split(t(y_log_lik_t_equals_1_traces)[-length(temperatures),], seq(length(temperatures)-1)), diff(temperatures)))
+	sum(mapply(FUN=function(y_lik, t_diff) { log(mean(exp(t_diff*y_lik-min(t_diff*y_lik))))+min(t_diff*y_lik) }, split(t(y_log_lik_t_equals_1_traces)[-length(temperatures),], seq(length(temperatures)-1)), diff(temperatures)))
 }
 
 #' Concatenate objects of class \code{BeviMed}
@@ -191,13 +194,14 @@ stack_BeviMeds <- function(...) {
 				setNames(nm=c(
 					"y_log_lik",
 					"y_log_lik_t_equals_1",
-					"Z",
+					"z",
+					"x",
 					"logit_omega",
 					"log_phi"
 				)),
 				function(trace_name) do.call(what=rbind, lapply(bevis, "[[", trace_name))
 			),
-			bevis[[length(bevis)]][-which(names(bevis[[length(bevis)]]) %in% c("y_log_lik", "y_log_lik_t_equals_1", "Z", "logit_omega", "log_phi"))]
+			bevis[[length(bevis)]][-which(names(bevis[[length(bevis)]]) %in% c("y_log_lik", "y_log_lik_t_equals_1", "z", "x", "logit_omega", "log_phi"))]
 		),
 		class="BeviMed"
 	)
@@ -239,15 +243,15 @@ CI_gamma1_evidence <- function(
 #' Sample blocks of a given size until either the estimated log marginal likelihood falls within a given confidence interval, there is sufficient confidence that the log Bayes factor of model gamma = 1 to model gamma = 0 is at most a certain quantity, or a certain number of blocks have been sampled.
 #' @template y
 #' @param blocks_remaining Maximum number of blocks left before termination.
-#' @param start_Zs Initial (logical) Z-matrix.
+#' @param start_zs Initial (logical) z-matrix.
 #' @param start_logit_omegas Initial values of logit_omega (numeric vector - one value per chain).
 #' @param start_log_phis Initial values of log_phi (numeric vector - one value per chain).
 #' @template temperatures
 #' @param tolerance Maximum width for confidence_interval of log marginal likelihood to allow before stopping the chain.
 #' @template confidence
 #' @template simulations
-#' @template gamma0_shape
-#' @param quit_if_highest_possible_BF_less_than Numeric value used to determine whether to stop the sampling after consecutive blocks. If we are confident (to the extent given by the parameter \code{confidence}) that log Bayes factor of v against n is under this value, we stop the sampling as soon as possible.
+#' @template tau0_shape
+#' @param BF_threshold Numeric value used to determine whether to stop the sampling procedure after successive blocks. If we are confident (to the level of \code{confidence}) that the log Bayes factor between models gamma = 1 and gamma = 0 is under this value, we stop the sampling as soon as possible.
 #' @template y_log_lik_t_equals_1_traces
 #' @param full_block_traces List of outputs of calls to MCMC routine. 
 #' @template verbose
@@ -256,15 +260,15 @@ CI_gamma1_evidence <- function(
 stop_chain <- function(
 	y,
 	blocks_remaining,
-	start_Zs,
+	start_zs,
 	start_logit_omegas,
 	start_log_phis,
 	temperatures,
 	tolerance=1,
 	confidence=0.95,
 	simulations=1000,
-	gamma0_shape=c(2, 100),
-	quit_if_highest_possible_BF_less_than=-Inf,
+	tau0_shape=c(1,1),
+	BF_threshold=-Inf,
 	y_log_lik_t_equals_1_traces=matrix(ncol=length(temperatures),nrow=0),
 	full_block_traces=list(),
 	verbose=FALSE,
@@ -273,7 +277,7 @@ stop_chain <- function(
 	if (verbose) cat("Sampling up to ", blocks_remaining, " more blocks to get marginal likelihood within tolerance of ", tolerance, "\n", sep="\n")
 
 	mc <- call_cpp(
-		Z0_matrix=start_Zs,
+		z0_matrix=start_zs,
 		logit_omegas=start_logit_omegas,
 		log_phis=start_log_phis,
 		temperatures=temperatures,
@@ -293,7 +297,7 @@ stop_chain <- function(
 		print(round(digits=2, confidence_interval))
 	}
 
-	if (diff(confidence_interval) < tolerance | (confidence_interval[2]-gamma0_evidence(y, shape=gamma0_shape)) < quit_if_highest_possible_BF_less_than | blocks_remaining <= 1) {
+	if (diff(confidence_interval) < tolerance | (confidence_interval[2]-gamma0_evidence(y, tau0_shape=tau0_shape)) < BF_threshold | blocks_remaining <= 1) {
 		do.call(what=stack_BeviMeds, c(full_block_traces, list(mc)))
 	} else {
 		stop_chain(
@@ -301,11 +305,11 @@ stop_chain <- function(
 			blocks_remaining=blocks_remaining - 1,
 			tolerance=tolerance,
 			confidence=confidence,
-			start_Zs=mc[["terminal_Z"]],
+			start_zs=mc[["terminal_z"]],
 			start_logit_omegas=mc[["terminal_logit_omega"]],
 			start_log_phis=mc[["terminal_log_phi"]],
 			temperatures=temperatures,
-			quit_if_highest_possible_BF_less_than=quit_if_highest_possible_BF_less_than,
+			BF_threshold=BF_threshold,
 			y_log_lik_t_equals_1_traces=y_ll,
 			full_block_traces=c(full_block_traces, list(mc)),
 			verbose=verbose,
@@ -379,8 +383,8 @@ tune_temperatures <- function(
 		largest <- which.max(areas)
 		t_pair <- c(largest, largest+1)
 		t_intersect <- mean(temperatures[t_pair])
-		temperatures <- c(temperatures[1:largest], t_intersect, temperatures[(largest+1):length(temperatures)])
-		chains <- c(chains[1:largest], list(call_cpp(temperatures=t_intersect, ...)), chains[(largest+1):length(chains)])
+		temperatures <- c(temperatures[seq(largest)], t_intersect, temperatures[(largest+1):length(temperatures)])
+		chains <- c(chains[seq(largest)], list(call_cpp(temperatures=t_intersect, ...)), chains[(largest+1):length(chains)])
 
 		E <- sapply(chains, function(ch) mean(ch[["y_log_lik_t_equals_1"]]))
 		V <- sapply(chains, function(ch) var(ch[["y_log_lik_t_equals_1"]]))
@@ -395,8 +399,8 @@ tune_temperatures <- function(
 #' @template y
 #' @template G_matrix
 #' @template min_ac
-#' @template q_shape
-#' @template p_shape
+#' @template tau_shape
+#' @template pi_shape
 #' @template omega_shape 
 #' @template samples_per_chain
 #' @param stop_early Logical value determining whether to attempt to stop the sampling as soon as certain conditions are met (i.e. either the estimated marginal log likelihood lies within a certain confidence interval, or we are sufficiently confidence that the log Bayes factor against of model gamma = 1 over model gamma = 0 is sufficiently low).
@@ -404,12 +408,14 @@ tune_temperatures <- function(
 #' @template burn
 #' @template temperatures
 #' @param tune_temps Integer value - if greater than 0, the \code{temperatures} argument is ignored, and instead \code{tune_temps} tuned temperatures are used instead.
-#' @template store_Z_trace
+#' @template return_z_trace
+#' @template return_x_trace
 #' @param swaps Number of swaps between adjacent tempered chains to perform per update cycle.
-#' @param optimise_Z0 Logical value determining whether to use a simulated annealing optimisation run to tune the initial values of \code{Z}.
+#' @param optimise_z0 Logical value determining whether to use a simulated annealing optimisation run to tune the initial values of \code{z}.
 #' @param tune_omega_and_phi_proposal_sd Logical value determining whether the proposal SDs of the Metropolis-Hastings estimated parameters should be tuned for a target acceptance range.
 #' @param tune_block_size Integer value giving number of samples to draw when estimatating the acceptance rate of the omega/phi proposals.
 #' @template variant_weights
+#' @param standardise_weights Boolean value determining whether weights should be standardised by subtracting their mean and dividing by their sample standard deviation. If \code{FALSE}, weights are untransformed.
 #' @template log_phi_mean
 #' @param log_phi_sd SD for normal prior on scaling factor phi. Setting to 0 causes the weights to be fixed and not estimated.
 #' @template tandem_variant_updates
@@ -421,44 +427,60 @@ bevimed <- function(
 	y,
 	G,
 	min_ac=1L,
-	q_shape=c(2,100),
-	p_shape=c(10, 2),
-	omega_shape=if (max(min_ac) == 1L) c(2, 9) else c(2, 2),
-	samples_per_chain=2000,
+	tau_shape=c(1, 1),
+	pi_shape=c(6, 1),
+	omega_shape=if (max(min_ac) == 1L) c(2, 8) else c(2, 2),
+	samples_per_chain=1000,
 	stop_early=FALSE,
 	blocks=5,
 	burn=as.integer(samples_per_chain/10),
-	temperatures=(0:10/10)^2,
+	temperatures=(0:6/6)^2,
 	tune_temps=0,
-	store_Z_trace=TRUE,
+	return_z_trace=TRUE,
+	return_x_trace=FALSE,
 	swaps=as.integer(length(temperatures)/2),
-	optimise_Z0=FALSE,
+	optimise_z0=FALSE,
 	tune_omega_and_phi_proposal_sd=FALSE,
 	tune_block_size=100,
 	variant_weights=NULL,
-	log_phi_mean=-1.5,
-	log_phi_sd=1,
+	standardise_weights=TRUE,
+	log_phi_mean=-0.15,
+	log_phi_sd=sqrt(0.3),
 	tandem_variant_updates=if (max(min_ac) == 1) 0 else min(sum(y), nrow(G)),
 	...
 ) {
 	stopifnot(is.matrix(G))
-	stopifnot(ncol(G)==length(y))
+	stopifnot(nrow(G)==length(y))
 	stopifnot(is.numeric(G))
 	stopifnot(is.logical(y))
+	stopifnot(min_ac > 0)
 
 	estimate_phi <- !((log_phi_sd == 0) | is.null(variant_weights))
 
-	normalised_weights <- (if (is.null(variant_weights) | (sd(variant_weights) == 0)) { rep(0L, nrow(G)) } else { (variant_weights - mean(variant_weights))/sd(variant_weights) }) * (if (estimate_phi) 1 else exp(log_phi_mean))
-
-	counts <- as.integer(t(G))
-	variants <- rep(1:nrow(G), each=ncol(G))
-	cases <- rep(1:ncol(G), times=nrow(G))
+	c_weights <- 
+		if (is.null(variant_weights)) { rep(0, ncol(G)) }
+		else {
+			stopifnot(is.numeric(variant_weights))
+			stopifnot(length(variant_weights) == ncol(G))
+			if (standardise_weights) {
+				if (length(variant_weights) == 1L | sd(variant_weights) == 0)
+					rep(0, ncol(G))
+				else
+					(variant_weights-mean(variant_weights))/sd(variant_weights)
+			} else {
+				variant_weights
+			}
+		} 
+		
+	counts <- as.integer(G)
+	variants <- rep(seq(ncol(G)), each=nrow(G))
+	cases <- rep(seq(nrow(G)), times=ncol(G))
 
 	block_ends <- cumsum(lapply(split(counts, variants), function(cnts) sum(cnts > 0)))
 	block_starts <- c(0, block_ends[-length(block_ends)])
 
-	y1_cases_with_more_than_1_variant <- apply(G > 0, 2, sum) > 1
-	y1_variants <- lapply(split(t(G[,y1_cases_with_more_than_1_variant] > 0), seq(sum(y1_cases_with_more_than_1_variant))), which)
+	y1_cases_with_more_than_1_variant <- apply(G > 0, 1, sum) > 1
+	y1_variants <- lapply(split(t(G[y1_cases_with_more_than_1_variant,,drop=FALSE] > 0), seq(sum(y1_cases_with_more_than_1_variant))), which)
 	y1_block_ends <- cumsum(lapply(y1_variants, length))
 	y1_block_starts <- c(0, y1_block_ends[-length(y1_block_ends)])
 	adjusted_tvu <- if (sum(y1_cases_with_more_than_1_variant) > 0) tandem_variant_updates else 0
@@ -474,11 +496,11 @@ bevimed <- function(
 		cases=cases[counts > 0]-1,
 		counts=counts[counts > 0],
 		min_ac=min_ac,
-		q_shape=q_shape,
-		p_shape=p_shape,
+		tau_shape=tau_shape,
+		pi_shape=pi_shape,
 		omega_shape=omega_shape,
 		estimate_omega=estimate_omega,
-		variant_weights=normalised_weights,
+		variant_weights=c_weights,
 		estimate_phi=estimate_phi,
 		log_phi_mean=log_phi_mean,
 		log_phi_sd=log_phi_sd,
@@ -498,24 +520,25 @@ bevimed <- function(
 					samples_per_chain=samples_per_chain,
 					number_of_temperatures=tune_temps,
 					return_temperatures=TRUE,
-					Z0_matrix=matrix(runif(nrow(G)) < omega_shape[1]/sum(omega_shape), nrow=1, ncol=nrow(G)),
+					z0_matrix=matrix(runif(ncol(G)) < omega_shape[1]/sum(omega_shape), nrow=1, ncol=ncol(G)),
 					logit_omegas=local({ w <- rbeta(n=1, shape1=omega_shape[1], shape2=omega_shape[2]); log(w)-log(1-w) }),
 					logit_omega_proposal_sds=rep(initial_logit_omega_proposal_sd, 1),
 					log_phis=if (estimate_phi) rnorm(n=1, mean=log_phi_mean, sd=log_phi_sd) else rep(0, 1),
 					log_phi_proposal_sds=rep(initial_log_phi_proposal_sd, 1), 
 					annealing=FALSE,
-					store_Z_trace=FALSE
+					return_z_trace=FALSE,
+					return_x_trace=FALSE
 				)
 			)
 		)
 	}
 
-	initial_Z <- if (optimise_Z0) {
+	initial_z <- if (optimise_z0) {
 		do.call(what=call_cpp, c(
 			reused_arguments,
 			list(
 				samples_per_chain=samples_per_chain,
-				Z0_matrix=matrix(runif(nrow(G)*length(temperatures)) < omega_shape[1]/sum(omega_shape), nrow=length(temperatures), ncol=nrow(G)),
+				z0_matrix=matrix(runif(ncol(G)*length(temperatures)) < omega_shape[1]/sum(omega_shape), nrow=length(temperatures), ncol=ncol(G)),
 				logit_omegas=local({ w <- rbeta(n=temperatures, shape1=omega_shape[1], shape2=omega_shape[2]); log(w)-log(1-w) }),
 				logit_omega_proposal_sds=rep(initial_logit_omega_proposal_sd, length(temperatures)),
 				log_phis=if (estimate_phi) rnorm(n=length(temperatures), mean=log_phi_mean, sd=log_phi_sd) else rep(0, length(temperatures)),
@@ -523,11 +546,12 @@ bevimed <- function(
 				temperatures=rep(1, length(temperatures)),
 				chain_swaps_per_cycle=swaps,
 				annealing=TRUE,
-				store_Z_trace=FALSE
+				return_z_trace=FALSE,
+				return_x_trace=FALSE
 			)
-		))[["terminal_Z"]]
+		))[["terminal_z"]]
 	} else {
-		matrix(runif(nrow(G) * length(temperatures)) < omega_shape[1]/sum(omega_shape), nrow=length(temperatures), ncol=nrow(G))
+		matrix(runif(ncol(G) * length(temperatures)) < omega_shape[1]/sum(omega_shape), nrow=length(temperatures), ncol=ncol(G))
 	}
 
 	proposal_sds <- lapply(
@@ -539,12 +563,13 @@ bevimed <- function(
 					initial_proposal_sds=rep(if (tune_for == "log_phi") initial_log_phi_proposal_sd else initial_logit_omega_proposal_sd, length(temperatures)),
 					samples_per_chain=tune_block_size,
 					burn=0,
-					Z0_matrix=initial_Z,
+					z0_matrix=initial_z,
 					logit_omegas=local({ w <- rbeta(n=length(temperatures), shape1=omega_shape[1], shape2=omega_shape[2]); log(w)-log(1-w) }),
 					log_phis=if (estimate_phi) rnorm(n=length(temperatures), mean=log_phi_mean, sd=log_phi_sd) else rep(0, length(temperatures)),
 					temperatures=temperatures,
 					annealing=FALSE,
-					store_Z_trace=FALSE
+					return_z_trace=FALSE,
+					return_x_trace=FALSE
 				),
 				list(...)[intersect(names(list(...)), names(formals(tune_proposal_sds)))]
 			))
@@ -558,14 +583,15 @@ bevimed <- function(
 			list(
 				samples_per_chain=samples_per_chain,
 				burn=0,
-				Z0_matrix=initial_Z,
+				z0_matrix=initial_z,
 				logit_omegas=local({ w <- rbeta(n=length(temperatures), shape1=omega_shape[1], shape2=omega_shape[2]); log(w)-log(1-w) }),
 				logit_omega_proposal_sds=proposal_sds[["logit_omega"]],
 				log_phis=if (estimate_phi) rnorm(n=length(temperatures), mean=log_phi_mean, sd=log_phi_sd) else rep(0, length(temperatures)),
 				log_phi_proposal_sds=proposal_sds[["log_phi"]], 
 				temperatures=temperatures,
 				annealing=FALSE,
-				store_Z_trace=FALSE
+				return_z_trace=FALSE,
+				return_x_trace=FALSE
 			)
 		))
 
@@ -575,7 +601,7 @@ bevimed <- function(
 				samples_per_chain=samples_per_chain,
 				y_log_lik_t_equals_1=matrix(ncol=length(temperatures),nrow=0),
 				burn=0,
-				start_Zs=burn[["terminal_Z"]],
+				start_zs=burn[["terminal_z"]],
 				start_logit_omegas=burn[["terminal_logit_omega"]],
 				start_log_phis=burn[["terminal_log_phi"]],
 				temperatures=temperatures,
@@ -583,7 +609,8 @@ bevimed <- function(
 				logit_omega_proposal_sds=proposal_sds[["logit_omega"]],
 				log_phi_proposal_sds=proposal_sds[["log_phi"]], 
 				annealing=FALSE,
-				store_Z_trace=store_Z_trace
+				return_z_trace=return_z_trace,
+				return_x_trace=return_x_trace
 			),
 			list(...)[intersect(names(list(...)), names(formals(stop_chain)))]
 		))
@@ -595,233 +622,36 @@ bevimed <- function(
 				list(
 					samples_per_chain=samples_per_chain,
 					burn=burn,
-					Z0_matrix=initial_Z,
+					z0_matrix=initial_z,
 					logit_omegas=local({ w <- rbeta(n=length(temperatures), shape1=omega_shape[1], shape2=omega_shape[2]); log(w)-log(1-w) }),
 					logit_omega_proposal_sds=proposal_sds[["logit_omega"]],
 					log_phis=if (estimate_phi) rnorm(n=length(temperatures), mean=log_phi_mean, sd=log_phi_sd) else rep(0, length(temperatures)),
 					log_phi_proposal_sds=proposal_sds[["log_phi"]], 
 					temperatures=temperatures,
 					annealing=FALSE,
-					store_Z_trace=store_Z_trace
+					return_z_trace=return_z_trace,
+					return_x_trace=return_x_trace
 				)
 			)
 		)
 	}
 }
 
-#' Calculate marginal probability of observed case-control status y under model gamma = 0
-#' 
-#' Marginal probability calculated exactly by integration.
-#' @template y
-#' @param shape Beta shape hyper-priors for prior on rate of case labels
-#' @export
-gamma0_evidence <- function(
-	y,
-	shape=c(2, 100)
-) {
-	lbeta(sum(y)+shape[1], length(y)-sum(y)+shape[2]) -
-	lbeta(shape[1], shape[2])
-}
-
-#' Calculate marginal probability of observed case-control status under model gamma = 1 
-#' 
-#' @template y
-#' @template G_matrix
-#' @template min_ac
-#' @param ... Other arguments to pass to \code{\link{bevimed}}.
-#' @export
-gamma1_evidence <- function(
-	y,
-	G,
-	min_ac=1L,
-	...
-) {
-	stopifnot(min(min_ac) > 0)
-	stopifnot(is.matrix(G) & is.numeric(G))
-	stopifnot(ncol(G) == length(y))
-
-	bv <- bevimed(
-		store_Z_trace=FALSE,
-		y=y,
-		G=G,
-		min_ac=min_ac,
-		...
-	)
-
-	sum_ML_over_PP(bv[["y_log_lik_t_equals_1"]], bv[["temperatures"]])
-}
-
-#' Calculate log Bayes factor between models gamma=1 and gamma=0 given the data
-#' 
-#' @template y
-#' @param ... Other arguments to pass to \code{\link{bevimed}}.
-#' @template gamma0_shape
-#' @return Log Bayes factor.
-#' @export
-log_BF <- function(
-	y,
-	gamma0_shape=c(2, 100),
-	...
-) {
-	gamma1_evidence(
-		y=y,
-		...
-	) - 
-	gamma0_evidence(y, shape=gamma0_shape)
-}
-
-#' Calculate probability of an association between presence/absence of local genotype configuration and case-control label
-#'
-#' @template y
-#' @template G_matrix
-#' @template ploidy 
-#' @template dominant_prior
-#' @template recessive_prior
-#' @template gamma0_shape
-#' @template by_MOI
-#' @param ... Other arguments to pass to \code{\link{log_BF}}. 
-#' @return Probability of association.
-#' @export
-prob_association <- function(
-	y,
-	G,
-	ploidy=rep(2L, length(y)),
-	dominant_prior=0.01,
-	recessive_prior=0.01,
-	gamma0_shape=c(2, 100),
-	by_MOI=FALSE,
-	...
-) {
-	stopifnot(is.matrix(G) & is.numeric(G))
-	stopifnot(ncol(G) == length(y))
-
-	dom_ev <- if (dominant_prior > 0) gamma1_evidence(y=y, G=G, min_ac=1L, ...) else -Inf
-	rec_ev <- if (recessive_prior > 0) gamma1_evidence(y=y, G=G, min_ac=ploidy, ...) else -Inf
-	g0_ev <- gamma0_evidence(y=y, shape=gamma0_shape)
-
-	modal_model <- max(c(dom_ev, rec_ev, g0_ev))
-	num_dom <- dominant_prior * exp(dom_ev-modal_model)
-	num_rec <- recessive_prior * exp(rec_ev-modal_model)
-	num_g0 <- (1-dominant_prior-recessive_prior) * exp(g0_ev-modal_model)
-	denom <- num_dom + num_rec + num_g0
-
-	#posterior
-	moi_probs <- c(Dominant=num_dom/denom, Recessive=num_rec/denom)
-	if (by_MOI) moi_probs
-	else sum(moi_probs)
-}
-
-#' Calculate probability of an association
-#'
-#'Synonym of \code{\link{prob_association}}
-#'
-#' @param ... Arguments passed to \code{\link{prob_association}}
-#' @return Probability of association.
-#' @export
-gamma1_prob <- function(...) prob_association(...)
-
-#' Calculate probability of pathogencity for variants in region given a prior probability of association between case label and the region
-#'
-#' @template y
-#' @template G_matrix
-#' @template ploidy 
-#' @template dominant_prior
-#' @template recessive_prior
-#' @template gamma0_shape
-#' @template by_MOI
-#' @param ... Other arguments to pass to \code{\link{bevimed}}. 
-#' @return Probabilities of pathogenicity.
-#' @export
-prob_pathogenic <- function(
-	y,
-	G,
-	ploidy=rep(2L, length(y)),
-	dominant_prior=0.01,
-	recessive_prior=0.01,
-	gamma0_shape=c(2, 100),
-	by_MOI=FALSE,
-	...
-) {
-	stopifnot(is.matrix(G) & is.numeric(G))
-	stopifnot(ncol(G) == length(y))
-
-	dom_bv <- bevimed(
-		store_Z_trace=TRUE,
-		y=y,
-		G=G,
-		min_ac=1L,
-		...
-	)
-
-	rec_bv <- bevimed(
-		store_Z_trace=TRUE,
-		y=y,
-		G=G,
-		min_ac=ploidy,
-		...
-	)
-
-	dom_ev <- sum_ML_over_PP(dom_bv[["y_log_lik_t_equals_1"]], dom_bv[["temperatures"]])
-	rec_ev <- sum_ML_over_PP(rec_bv[["y_log_lik_t_equals_1"]], rec_bv[["temperatures"]])
-	g0_ev <- gamma0_evidence(y, shape=gamma0_shape)
-
-	modal_model <- max(c(dom_ev, rec_ev, g0_ev))
-	num_dom <- dominant_prior * exp(dom_ev-modal_model)
-	num_rec <- recessive_prior * exp(rec_ev-modal_model)
-	num_g0 <- (1-dominant_prior-recessive_prior) * exp(g0_ev-modal_model)
-	denom <- num_dom + num_rec + num_g0
-
-	dom_conditional_prob_pathogenic <- apply(dom_bv[["Z"]][,dom_bv[["k"]]*(length(dom_bv[["temperatures"]])-1)+1:dom_bv[["k"]],drop=FALSE], 2, mean)
-	rec_conditional_prob_pathogenic <- apply(rec_bv[["Z"]][,rec_bv[["k"]]*(length(rec_bv[["temperatures"]])-1)+1:rec_bv[["k"]],drop=FALSE], 2, mean)
-
-	prob_by_moi <- cbind(
-		Dominant=dom_conditional_prob_pathogenic*num_dom/denom,
-		Recessive=rec_conditional_prob_pathogenic*num_rec/denom
-	)
-	if (by_MOI) prob_by_moi
-	else apply(prob_by_moi, 1, sum)
-}
-
-#' Calculate probability of pathogencity for variants in region given an association between case label and the region
-#'
-#' @template y
-#' @template G_matrix
-#' @template min_ac
-#' @param ... Other arguments to pass to \code{\link{bevimed}}. 
-#' @return Probabilities of pathogenicity.
-#' @export
-conditional_prob_pathogenic <- function(
-	y,
-	G,
-	min_ac=1L,
-	...
-) {
-	bv <- bevimed(
-		store_Z_trace=TRUE,
-		y=y,
-		G=G,
-		min_ac=min_ac,
-		...
-	)
-
-	apply(bv[["Z"]][,bv[["k"]]*(length(bv[["temperatures"]])-1)+1:bv[["k"]],drop=FALSE], 2, mean)
-}
-
 #' Calculate marginal probability of observed genotypes under 'pathogenic region' model
 #' 
 #' Marginal probability calculated exactly by integration.
 #' @template y
 #' @param G Integer/logical vector of genotypes by individual corresponding to case-control label \code{y} giving the 'rare variant counts'/'presence of rare variant indicators'.
 #' @template min_ac
-#' @template q_shape
-#' @template p_shape
+#' @template tau_shape
+#' @template pi_shape
 #' @export
 region_association_evidence <- function(
 	y,
 	G,
 	min_ac=1L,
-	q_shape=c(2, 4),
-	p_shape=c(1, 1)
+	tau_shape=c(1, 1),
+	pi_shape=c(2, 1)
 ) {
 	stopifnot(is.vector(G))
 	stopifnot(length(y) == length(G))
@@ -830,24 +660,24 @@ region_association_evidence <- function(
 	`G'` <- if (is.logical(G)) G else G >= min_ac
 
 	ll_pathogenic <- 
-		+ lbeta(sum(y[`G'`])+p_shape[1], sum(`G'`)-sum(y[`G'`])+p_shape[2])
-		- lbeta(p_shape[1], p_shape[2])
+		+ lbeta(sum(y[`G'`])+pi_shape[1], sum(`G'`)-sum(y[`G'`])+pi_shape[2])
+		- lbeta(pi_shape[1], pi_shape[2])
 
 	ll_benign <- 
-		+ lbeta(sum(y[!`G'`])+q_shape[1], sum(!`G'`)-sum(y[!`G'`])+q_shape[2])
-		- lbeta(q_shape[1], q_shape[2])
+		+ lbeta(sum(y[!`G'`])+tau_shape[1], sum(!`G'`)-sum(y[!`G'`])+tau_shape[2])
+		- lbeta(tau_shape[1], tau_shape[2])
 
 	ll_pathogenic+ll_benign
 }
 
-#' Calculate log lower bound for marginal probability of observations under model gamma = 1 by summing likelihood over pathogenic variant (Z) configurations, or probabilities that individual variants are pathogenic.
+#' Calculate log lower bound for marginal probability of observations under model gamma = 1 by summing likelihood over pathogenic variant (z) configurations, or probabilities that individual variants are pathogenic.
 #' 
 #' @template y
 #' @template G_matrix
 #' @template min_ac
 #' @param by_term Calculate probability that individual terms are pathogenic conditional on model gamma=1.
-#' @template q_shape
-#' @template p_shape
+#' @template tau_shape
+#' @template pi_shape
 #' @template omega_shape
 #' @param sum_over_variants Subset of variants for whose power set to calculate the direct sum over.
 #' @export
@@ -856,38 +686,38 @@ lower_bound_gamma1_evidence <- function(
 	G,
 	min_ac=1L,
 	by_term=FALSE,
-	q_shape=c(2,100),
-	p_shape=c(10, 2),
-	omega_shape=c(2, 9),
-	sum_over_variants=1:nrow(G)
+	tau_shape=c(1, 1),
+	pi_shape=c(6, 1),
+	omega_shape=c(2, 8),
+	sum_over_variants=seq(ncol(G))
 ) {
-	Z_confs <- as.matrix(do.call(what=expand.grid, rep(list(c(F,T)), length(sum_over_variants))))
+	z_confs <- as.matrix(do.call(what=expand.grid, rep(list(c(F,T)), length(sum_over_variants))))
 
 	log_pr_by_var <- apply(
-		Z_confs,
+		z_confs,
 		1,
-		function(Z) {
-			x_i <- apply(G[sum_over_variants[Z],,drop=FALSE], 2, function(variant_config) sum(variant_config)) >= min_ac
+		function(z) {
+			x_i <- apply(G[,sum_over_variants[z],drop=FALSE], 1, function(variant_config) sum(variant_config)) >= min_ac
 			sig11 <- sum(x_i & y)
 			sig10 <- sum(x_i & !y)
 			sig01 <- sum(!x_i & y)
 			sig00 <- sum(!x_i & !y)
 
 			(
-				+ lbeta(sig11+p_shape[1], sig10+p_shape[2])
-				+ lbeta(sig01+q_shape[1], sig00+q_shape[2])
-				+ lbeta(sum(Z)+omega_shape[1], nrow(G)-sum(Z)+omega_shape[2])
+				+ lbeta(sig11+pi_shape[1], sig10+pi_shape[2])
+				+ lbeta(sig01+tau_shape[1], sig00+tau_shape[2])
+				+ lbeta(sum(z)+omega_shape[1], ncol(G)-sum(z)+omega_shape[2])
 			) -
 			(
-			 	+ lbeta(p_shape[1], p_shape[2])
-			 	+ lbeta(q_shape[1], q_shape[2])
+			 	+ lbeta(pi_shape[1], pi_shape[2])
+			 	+ lbeta(tau_shape[1], tau_shape[2])
 			 	+ lbeta(omega_shape[1], omega_shape[2])
 			)
 		}
 	)
 
 	if (by_term) {
-		apply(Z_confs, 2, function(x) sum(exp(log_pr_by_var[x]-max(log_pr_by_var))))/sum(exp(log_pr_by_var-max(log_pr_by_var)))
+		apply(z_confs, 2, function(x) sum(exp(log_pr_by_var[x]-max(log_pr_by_var))))/sum(exp(log_pr_by_var-max(log_pr_by_var)))
 	} else {
 		log(sum(exp(log_pr_by_var-max(log_pr_by_var))))+max(log_pr_by_var)
 	}
@@ -899,5 +729,5 @@ lower_bound_gamma1_evidence <- function(
 #' @param ... Other arguments to pass to \code{\link{lower_bound_gamma1_evidence}}.
 #' @export
 exact_evidence <- function(G, ...) {
-	lower_bound_gamma1_evidence(G=G, sum_over_variants=1:nrow(G), ...)
+	lower_bound_gamma1_evidence(G=G, sum_over_variants=seq(ncol(G)), ...)
 }
